/*
 * Copyright (C) 2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IPC_HPP_
#define IPC_HPP_

#include "compiler.hpp"

#include <atomic>
#include <cstdint>
#include <ctime>
#include <unistd.h>

#define IPC_URI_MAX 256     // NOLINT

typedef enum { // NOLINT
    IPC_STATUS,
    IPC_LEVEL,      // verbosity
    IPC_RELOAD,
}   ipc_type_t;

struct ipc_event {
    ipc_type_t type;
    pid_t pid;
    union {
        unsigned level;     // logging level
    } body;
};

struct ipc_session {
    time_t created;
    char sid[10];
    char caller[32];
    char dialed[32];
    enum {UNKNOWN = 0, INVITING, RINGING, CONNECTED, CLOSING} state;
    enum {ANY = 0, INBOUND, OUTBOUND} type;
};

struct ipc_system {
    std::atomic<unsigned> used;
    time_t started;
    unsigned level, limit;
    char identity[IPC_URI_MAX];
    bool active;    // registered...
};

/*!
 * Use bordeaux ipc services. IPC includes shared memory blocks for the
 * registry and call processing, and a control mqueue. This header may be
 * be possible to include in stand-alone external third party projects
 * as well. Because ipc does not cross multiple machines, native byte order
 * and struct alignment can be used.
 * \file ipc.hpp
 */
#endif
