/*
 * Copyright (C) 2020-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PROC_HPP
#define PROC_HPP

#include "compiler.hpp"

#include <string>
#include <memory>
#include <list>

#include <unistd.h>

namespace server {
extern uid_t nobody;
class Session;
using session_t = std::shared_ptr<Session>;
} // end server namespace

namespace server::proc {
void ipc_notify(pid_t pid, bool success);
auto create(session_t& session, std::string argv0, std::list<std::string> args, int stdin = -1, int stdout = -1, std::list<std::string> env = {}) -> pid_t;
void drop(pid_t pid, int status);
void stop(pid_t pid);
void hangup(pid_t pid);
void killall();
auto get_priority() -> int;
auto set_priority(int priority) -> bool;
} // end namespace

/*!
 * Process functions and support for server.
 * \file proc.hpp
 */
#endif
