/*
 * Copyright (C) 2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTILS_HPP_
#define UTILS_HPP_

#include "compiler.hpp"

#include <list>
#include <string>

namespace server {
class dir final
{
public:
    dir(const dir&) = delete;
    auto operator=(const dir&) -> dir& = delete;

    explicit dir(const std::string& path) noexcept;
    dir(dir&& from) noexcept;
    dir() noexcept;
    ~dir();

    inline explicit operator bool() const {
        return state != nullptr;
    }

    inline auto operator!() const {
        return state == nullptr;
    }

    inline auto operator*() -> std::string {
        return get();
    }

    auto operator=(dir&& from) noexcept -> dir&;
    void scan(const std::string& path);
    auto get() -> std::string;

private:
    struct stateinfo;   // mechanics hidden...
    struct stateinfo *state;

    void release();
    void next();
};
void logger(const std::string& msg);
void log_reset();
} // end namespace server

namespace server::util {
inline auto get8(const uint8_t *ptr) {
    return *ptr;
}

inline void put8(uint8_t *ptr, uint8_t value) {
    *ptr = value;
}

auto get16(const uint8_t *ptr) -> uint16_t;
auto get32(const uint8_t *ptr) -> uint32_t;
auto get64(const uint8_t *ptr) -> uint64_t;
void set16(uint8_t *ptr, uint16_t value);
void set32(uint8_t *ptr, uint32_t value);
void set64(uint8_t *ptr, uint64_t value);

auto get_env(const char *id, size_t max = FILENAME_MAX) -> const char *;
auto u8verify(const char *cp, size_t max = 255) -> size_t;
auto get_value(std::string_view text) -> int;
auto compare(std::string_view from, std::string_view to) -> bool;
auto dirname(const std::string& path) -> std::string;
auto basename(const std::string& path, std::initializer_list<std::string_view> extensions = {}) -> std::string;
auto exists(const std::string& path) -> bool;
auto is_dir(const std::string& path) -> bool;
auto to_b64(const uint8_t *data, size_t size) -> std::string;
auto random_fill(uint8_t *data, size_t size) -> bool;
auto set_string(char *cp, size_t max, std::string_view view) -> size_t;
auto count(const char *cp, size_t max = 256) -> size_t;
} // end namespace server utils

/*!
 * Utility functions and support for the server.
 * \file utils.hpp
 */
#endif
